import java.util.StringTokenizer;
import java.io.*;
import java.sql.*;
import java.awt.BorderLayout;
import java.awt.Cursor;
import javax.swing.*;
import java.awt.event.*;

public class DataTransfer
{   private DataViewer parent = null;

    private JList tables = null;
    private DefaultListModel tableData = null;
    private JTextField filter = null;

    private String extension = ".sql.csv";
    private String delim = ",";

    public DataTransfer(DataViewer parent)
    {   this.parent = parent;
    }

    public void importData(String tableName)
    {   JFileChooser chooser = new JFileChooser();
        chooser.setDialogTitle("Choose file for import into " + tableName);
        chooser.setDialogType(JFileChooser.OPEN_DIALOG);
        int returnVal = chooser.showOpenDialog(parent);
        if(returnVal == JFileChooser.APPROVE_OPTION) 
        {   File file = chooser.getSelectedFile();
            try
            {   BufferedReader in = new BufferedReader(
                                       new FileReader(file));
                Connection connection = parent.getConnection();
                Statement statement = connection.createStatement();
    
                String line = new String();
                int rowsInserted = 0;

                while (line != null)
                {   line = in.readLine();
                    StringBuffer sb = new StringBuffer("insert into " + tableName + " values (");
                    if (line != null)
                    {   StringTokenizer tokens = new StringTokenizer(line, delim);
                        while (tokens.hasMoreTokens())
                        {   sb.append(tokens.nextToken());
                            sb.append(",");
                        }
                        sb.deleteCharAt(sb.length()-1);
                        sb.append(")");
                        rowsInserted += statement.executeUpdate(sb.toString());
                    }
                }
                in.close();
                JOptionPane.showMessageDialog(parent, "Successfully imported " + rowsInserted + " rows into " + tableName);
            }
            catch(IOException ioe)
            {   JOptionPane.showMessageDialog(parent, ioe.getMessage());
            }
            catch(SQLException sqle)
            {   JOptionPane.showMessageDialog(parent, sqle.getMessage());
            }
        }
    }

    public void exportData(DefaultListModel tableData, DatabaseMetaData dbInfo)
    {   String tableName = tableData.get(0).toString();

        int returnValue = JOptionPane.showConfirmDialog(parent, 
                                                        exportOptionsPanel(tableData), 
                                                        "Export Options", 
                                                        JOptionPane.OK_CANCEL_OPTION);

        if (returnValue == JOptionPane.OK_OPTION) 
        {   Object tableNames[] = tables.getSelectedValues();
            if ((tableNames == null) || (tableNames.length == 0))
                JOptionPane.showMessageDialog(parent, "No table(s) selected.");
            else
            {   parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                for (int i = 0; i < tableNames.length; i++)
                    exportTable(tableNames[i].toString(), filter.getText().trim());
                parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
        }
    }

    private void exportTable(String tableName, String filter)
    {   File file = new File(tableName + extension);
        try
        {
            PrintWriter out = new PrintWriter(
                                  new FileWriter(file));
            Connection connection = parent.getConnection();
            Statement statement = connection.createStatement();
            String sqlSelect = "select * from " + tableName;
            if (!filter.equals(""))
                sqlSelect += " where (" + filter + ")";
            ResultSet results = statement.executeQuery(sqlSelect);
            ResultSetMetaData resultsInfo = results.getMetaData();

            int numColumns = resultsInfo.getColumnCount();
            int rowCount = 0;

            while (results.next())
            {   StringBuffer sb = new StringBuffer();
                for (int i = 1; i <= numColumns; i++)
                {   if (DataViewerData.useQuotes(resultsInfo, i))
                    {   sb.append("'");
                        sb.append(results.getString(i).trim());
                        sb.append("'");
                    }
                    else
                        sb.append(results.getString(i).trim());
                    sb.append(delim);
                }
                sb.deleteCharAt(sb.length()-1);
                out.println(sb.toString());
                rowCount++;
            }
            results.close();
            out.close();
            JOptionPane.showMessageDialog(parent, rowCount + " rows successfully exported to " + file.getName());
        }
        catch(IOException ioe)
        {   JOptionPane.showMessageDialog(parent, "IO Error exporting " + tableName + " (" + ioe.getMessage() + ")");
        }
        catch(SQLException sqle)
        {   JOptionPane.showMessageDialog(parent, "SQL Error exporting " + tableName + " (" + sqle.getMessage() + ")");
        }
    }

    private JPanel exportOptionsPanel(DefaultListModel tableData)
    {   this.tableData = tableData;
        tables = new JList(tableData);
        filter = new JTextField();
        JPanel panel = new JPanel(new BorderLayout());
        panel.add("Center", new JPanelBox(new JScrollPane(tables), "Select table(s)"));
        panel.add("South", new JPanelBox(filter, "Enter SQL filter (optional)"));
        return panel;
    }
}