import java.sql.*;
import java.net.*;
import java.awt.*;
import java.util.Vector;
import java.util.Properties;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.border.*;

public class DataViewer extends JFrame implements ActionListener, ListSelectionListener
{   private static Font buttonFont = new Font("SansSerif", Font.PLAIN, 10);

    private String host = "localhost";
   private String data = "itd";
   private String user = "username";
   private String pswd = "password";
   private String port = "####";
   private int    driver = 0;
   private JTextField statement = new JTextField(50);
   private JButton getTables = new JButton("Open", new ImageIcon("icons/gettables.gif"));
   private JButton getData = new JButton("Show", new ImageIcon("icons/getfields.gif"));
   private JButton execute = new JButton("Execute", new ImageIcon("icons/execute2.gif"));
   private DefaultListModel tableData = new DefaultListModel();
   private DefaultListModel fieldData = new DefaultListModel();
   private JList   tableList = new JList(tableData);
   private JList   fieldList = new JList(fieldData);
   private SQLTableModel model = null;
   private JTable  table = null;
   private Connection connection = null;
   private DatabaseMetaData dbInfo = null;
   private DataViewerMenu menu = null;
   private DataTransfer dataTransfer = null;
   private class WindowCloser extends WindowAdapter
   {  public void windowClosing(WindowEvent we)
      {  exit();
      }
   }
   /* ****************************************************************************** */
   public DataViewer()
   {  super("JDBC Data Viewer");
      JPanel buttons = new JPanel(new FlowLayout());
      buttons.add(getTables);  getTables.setToolTipText("Connect to SQL Server");
      buttons.add(getData);    getData.setToolTipText("Get data from SQL Server");
      buttons.add(execute);    execute.setToolTipText("Execute SQL statement");
      JPanel executor = new JPanel(new BorderLayout());
      executor.add("West", buttons); 
      executor.add("Center", statement);
      tableList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      JPanel lists = new JPanel(new GridLayout(1,2));
      lists.add(new JPanelBox(new JScrollPane(tableList), "Tables"));
      lists.add(new JPanelBox(new JScrollPane(fieldList), "Fields"));
      model = new SQLTableModel();
      table = new JTable(model);
      table.setPreferredScrollableViewportSize(new Dimension(450, 200));
      JSplitPane splitter = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true);
      splitter.setDividerSize(10);
      splitter.setOneTouchExpandable(true);
      splitter.setTopComponent(new JPanelBox(new JScrollPane(table), "Data Values"));
      splitter.setBottomComponent(lists);
      getContentPane().setLayout(new BorderLayout());
      getContentPane().add("North",  new JPanelBox(executor, "Execute SQL Statement"));
      getContentPane().add("Center",  splitter);
      getTables.addActionListener(this);
      getData.addActionListener(this);
      execute.addActionListener(this);
      statement.addActionListener(this);
      tableList.addListSelectionListener(this);
      addWindowListener(new WindowCloser());
        menu = new DataViewerMenu(this);
        dataTransfer = new DataTransfer(this);

      setJMenuBar(menu);

        getTables.setFont(buttonFont);
        getData.setFont(buttonFont);
        execute.setFont(buttonFont);

        getTables.setToolTipText("Connect to SQL server");
        getData.setToolTipText("Show data in selected table/fields");
        execute.setToolTipText("Execute SQL statement");

      validate(); pack(); setVisible(true);
   }
   /* ****************************************************************************** */
   public void exit()
   {  try
      {  disconnect();
      }
      catch(SQLException sqle)
      {  System.err.println("Error closing connection: " + sqle.getMessage());
      }
      finally
      {  System.exit(0);
      }
   }
   /* ****************************************************************************** */
   public Connection getConnection() throws SQLException
   {  if (connection != null)
         return connection;
      else
         throw new SQLException("No connection open");
   }
   /* ****************************************************************************** */
   public void addNewRow()
   {  try
      {  ResultSetMetaData info = model.getResults().getMetaData();
         String tableName = tableList.getSelectedValue().toString();
         DataViewerData data = new DataViewerData(this, info, tableName);
      }
      catch(SQLException sqle)
      {  JOptionPane.showMessageDialog(this, "<html><h3>Error: SQL Execution Error:</h3>" + sqle.getMessage() + "</html>", "SQL Error", JOptionPane.ERROR_MESSAGE); 
      }
   }
   /* ****************************************************************************** */
   public void editRow()
   {  try
      {  ResultSetMetaData info = model.getResults().getMetaData();
         String tableName = tableList.getSelectedValue().toString();
         Vector rowData = model.getRow(table.getSelectedRow());
         DataViewerData data = new DataViewerData(this, info, tableName, rowData);
      }
      catch(SQLException sqle)
      {  JOptionPane.showMessageDialog(this, "<html><h3>Error: SQL Execution Error:</h3>" + sqle.getMessage() + "</html>", "SQL Error", JOptionPane.ERROR_MESSAGE); 
      }
   }
   /* ****************************************************************************** */
   public void executeStatement(String SQLStatement)
   {  if (statement.getText().equals(""))
         JOptionPane.showMessageDialog(this, "Please enter valid SQL statement first","SQL Message", JOptionPane.WARNING_MESSAGE);
      else
      {  try
         {
            Statement stmt = connection.createStatement();
            if (stmt.execute(SQLStatement))
               model.setResults(stmt.getResultSet());
            else
               JOptionPane.showMessageDialog(this, "Execution effected " + stmt.getUpdateCount() + " rows.","SQL Message", JOptionPane.INFORMATION_MESSAGE);
         }
         catch(SQLException sqle)
         {  JOptionPane.showMessageDialog(this, "<html><h3>Error: SQL Execution Error:</h3>" + sqle.getMessage() + "</html>", "SQL Error", JOptionPane.ERROR_MESSAGE); 
         }
      }
   }
   /* ****************************************************************************** */
   public String constructQuery(String tableName, Object[] fieldNames)
   {  String fields = "*";
      if (fieldNames.length > 0)
      {  fields = "";
         for (int i = 0; i < fieldNames.length-1; i++)
            fields += fieldNames[i].toString().substring(0, fieldNames[i].toString().indexOf(" ")) + ",";
         fields += fieldNames[fieldNames.length-1].toString().substring(0, fieldNames[fieldNames.length-1].toString().indexOf(" "));
      }
      return "SELECT " + fields + " FROM " + tableName;
   }
   /* ****************************************************************************** */
   public void showData()
   {  if (tableList.getSelectedIndex() < 0)
         JOptionPane.showMessageDialog(this, "Please select a table first","SQL Notice", JOptionPane.WARNING_MESSAGE);
      else
      {  String query = constructQuery(tableList.getSelectedValue().toString(), fieldList.getSelectedValues());
         statement.setText(query);
         executeStatement(query);
      }
   }
   /* ****************************************************************************** */
   public void showFields(String tableName)
   {  try
      {
         fieldData.clear();
         table.clearSelection();
         ResultSet columns = dbInfo.getColumns(null, null, tableName, null);
         while (columns.next())
         {  try
            {
               fieldData.addElement(columns.getString("COLUMN_NAME") + " [" + columns.getString("TYPE_NAME") + "]");
            }
            catch(SQLException sqle)
            {  System.err.println("Can not get column information");
            }
         }
         columns.close();
         showData();
//         disconnect();
      }
      catch(SQLException sqle)
      {  JOptionPane.showMessageDialog(this, "<html><h3>Error: SQL error getting columns:</h3>" + sqle.getMessage() + "</html>", "SQL Error", JOptionPane.ERROR_MESSAGE); }
   }
   /* ****************************************************************************** */
   public void showTables()
   {  DataInfo info = new DataInfo(host, data, user, pswd, port);
      if (JOptionPane.showOptionDialog(this, info, DataInfo.TITLE,
                                       JOptionPane.DEFAULT_OPTION, 
                                       JOptionPane.INFORMATION_MESSAGE,
                                       null, DataInfo.BUTTONS, DataInfo.BUTTONS[0]) == 0) 
      {  host = info.getInfo()[0]; 
         data = info.getInfo()[1];
         user = info.getInfo()[2]; 
         pswd = info.getInfo()[3];
         port = info.getInfo()[4];
         driver = info.getDriverIndex();
         try
         {  connect();
            tableData.clear();
            fieldData.clear();
            table.clearSelection();
            ResultSet tables = dbInfo.getTables(null, null, null, null);
            while (tables.next())
            {  if (tables.getString("TABLE_TYPE").equals("TABLE"))
                  tableData.addElement(tables.getString("TABLE_NAME"));
            }
            tables.close();
         }
         catch(SQLException sqle)
         {  JOptionPane.showMessageDialog(this, "<html><h3>Error: SQL error showing tables:</h3>" + sqle.getMessage() + "</html>", "SQL Error", JOptionPane.ERROR_MESSAGE); }
      }
   }
   /* ****************************************************************************** */
    public void connect() throws SQLException
    {   
//        String connectionURL = "jdbc:db2://" + host + ":" + port + "/" + data;
        String connectionURL = DataInfo.DRIVER_PREFACE[driver] + host + ":" + port + "/" + data;
        try
        {   Class.forName(DataInfo.DRIVER_CLASS[driver]);
//            Class.forName("COM.ibm.db2.jdbc.net.DB2Driver");
            connection = DriverManager.getConnection(connectionURL, user, pswd);
            dbInfo     = connection.getMetaData();
            menu.setTransferEnabled(true);
            menu.setConnected(true);
            StringBuffer sb = new StringBuffer("<html>");
            sb.append("<h3>Connection Info</h3>");
            sb.append("<dl>");
            sb.append("<dt>DB Product:<dd>" + dbInfo.getDatabaseProductName());
            sb.append("<dt>DB Version: <dd>" + dbInfo.getDatabaseProductVersion());
            sb.append("<dt>supports entry SQL:<dd>" + dbInfo.supportsANSI92EntryLevelSQL());
            sb.append("<dt>supports medium SQL:<dd>" + dbInfo.supportsANSI92IntermediateSQL());
            sb.append("<dt>supports full SQL:<dd>" + dbInfo.supportsANSI92FullSQL());
            sb.append("</dl>");
            sb.append("</html>");
            JOptionPane.showMessageDialog(this, sb.toString());   
        }
        catch(ClassNotFoundException cnfe) 
        {   JOptionPane.showMessageDialog(this, "<html><H3>Error: Invalid SQL driver or driver not found</h3></html>","SQL Error", JOptionPane.ERROR_MESSAGE);
            connection = null;
        }
    }
   /* ****************************************************************************** */
    public void disconnect() throws SQLException
    {   if (connection != null)
        {   connection.close();
            JOptionPane.showMessageDialog(this, "Database connection terminated");
        }
        connection = null;
        fieldData.clear();
        tableData.clear();
        menu.setTransferEnabled(false);
        menu.setConnected(false);
    }
   /* ****************************************************************************** */
    public void importData()
    {   if (tableList.getSelectedIndex() < 0)
            JOptionPane.showMessageDialog(this, "Please select a table first","SQL Notice", JOptionPane.WARNING_MESSAGE);
        else
           dataTransfer.importData(tableList.getSelectedValue().toString());
    }
   /* ****************************************************************************** */
    public void exportData()
    {   if (tableData.size() == 0)
            JOptionPane.showMessageDialog(this, "The database contains no tables to export","SQL Notice", JOptionPane.WARNING_MESSAGE);
        else
           dataTransfer.exportData(tableData, dbInfo);
    }
   /* ****************************************************************************** */
   public void actionPerformed(ActionEvent ae)
   {  if (ae.getSource() == getTables)
         showTables();
      else if (ae.getSource() == getData)
         showData();
      else if ((ae.getSource() == execute) || (ae.getSource() == statement))
         executeStatement(statement.getText());
   }
   /* ****************************************************************************** */
   public void valueChanged(ListSelectionEvent le)
   {  if (!le.getValueIsAdjusting())
         showFields(tableData.elementAt(tableList.getSelectedIndex()).toString());
   }
   /* ****************************************************************************** */
   public static void main(String args[])
   {  try 
      {  
//         UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
         UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
//         UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
//         UIManager.setLookAndFeel("javax.swing.plaf.mac.MacLookAndFeel");
      }
      catch (Exception e) 
      { }
      DataViewer dbv = new DataViewer();
   }
   /* ****************************************************************************** */
}
