import java.util.Vector;
import java.sql.*;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.*;
import javax.swing.*;

public class DataViewerData extends JDialog implements ActionListener
{  private DataViewer parent;
   private ResultSetMetaData info;

   private JButton okay = new JButton("Okay");
   private JButton apply = new JButton("Apply");
   private JButton cancel = new JButton("Cancel");

   private JLabel       tableName;
   private JTextField[] columnFields;
   private JLabel[]     columnLabels;
   private JLabel[]     columnTypes;
   private boolean[]    quoted;
   private int          numCols;

   public DataViewerData(DataViewer parent, ResultSetMetaData info, String table)
   {  super(parent, "Data", true);

      this.parent = parent;
      this.info = info;
      tableName = new JLabel(table);
 
      setup();
   }
   public DataViewerData(DataViewer parent, ResultSetMetaData info, String table, Vector data)
   {  this(parent, info, table);
System.out.println("Setting data");
      for (int i = 0; i < numCols; i++)
         columnFields[i].setText(data.get(i).toString());
   }
   private void setup()
   {  JPanel buttons = new JPanel(new FlowLayout());
      buttons.add(okay);
      buttons.add(apply);
      buttons.add(cancel);

      setupRow();

      JPanel labels = new JPanel(new GridLayout(numCols, 1));
      JPanel fields = new JPanel(new GridLayout(numCols, 1));
      JPanel types = new JPanel(new GridLayout(numCols, 1));
      for (int i = 0; i < numCols; i++)
      {  labels.add(columnLabels[i]);
         fields.add(columnFields[i]);
         types.add(columnTypes[i]);
      }
      JPanel input = new JPanel(new BorderLayout());
      input.add("West", labels);
      input.add("Center", fields);
      input.add("East", types);

      getContentPane().setLayout(new BorderLayout());
      getContentPane().add("North", tableName);
      getContentPane().add("Center", input);
      getContentPane().add("South", buttons);

      okay.addActionListener(this);
      apply.addActionListener(this);
      cancel.addActionListener(this);

      validate();
      pack();
      show();
   }

   private void setupRow()
   {  try
      {  numCols = info.getColumnCount();
         quoted = new boolean[numCols];
         columnFields = new JTextField[numCols];
         columnLabels = new JLabel[numCols];
         columnTypes = new JLabel[numCols];
         for (int i = 1; i <= numCols; i++)
         {  columnTypes[i-1] = new JLabel(info.getColumnTypeName(i));
            columnLabels[i-1] = new JLabel(info.getColumnName(i));
            columnFields[i-1] = new JTextField(15);
            if (useQuotes(info, i))
               quoted[i-1] = true;
            else
               quoted[i-1] = false;
         }
      }
      catch(SQLException sqle)
      {  JOptionPane.showMessageDialog(parent, sqle.getMessage());
      }
   }

   private void doApply()
   {  StringBuffer query = new StringBuffer();
      query.append("INSERT INTO " + tableName.getText() + " VALUES (");
      for (int i = 0; i < numCols; i++)
      {  String data = columnFields[i].getText();
         if (quoted[i])
            query.append("'" + data + "'");
         else
            query.append(data);
         if (i != numCols-1)
            query.append(",");
         else
            query.append(")");
      }
      try
      {  Statement stmt = parent.getConnection().createStatement();
         int rows = stmt.executeUpdate(query.toString());
         JOptionPane.showMessageDialog(parent, "Rows added: " + rows);
      }
      catch(SQLException sqle)
      {  JOptionPane.showMessageDialog(parent, "Error: " + sqle.getMessage());
      }
   }

   public void actionPerformed(ActionEvent ae)
   {  if (ae.getSource() == okay)
      {  doApply();
         hide();
         dispose();
      }
      else if (ae.getSource() == apply)
      {  doApply();
      }
      else if (ae.getSource() == cancel)
      {  hide();
         dispose(); 
      }
   }
    public static boolean useQuotes(ResultSetMetaData info, int col) throws SQLException
    {   int type = info.getColumnType(col);
        if ((type == Types.ARRAY) ||
            (type == Types.BINARY) ||
            (type == Types.BIT) ||
            (type == Types.BLOB) ||
            (type == Types.CLOB) ||
            (type == Types.JAVA_OBJECT) ||
            (type == Types.LONGVARBINARY) ||
            (type == Types.OTHER) ||
            (type == Types.REF) ||
            (type == Types.STRUCT) ||
            (type == Types.VARBINARY))
            throw new SQLException("Type not supported for export, sorry.");
        if ((type == Types.BIGINT) ||
            (type == Types.DECIMAL) ||
            (type == Types.DOUBLE) ||
            (type == Types.FLOAT) ||
            (type == Types.INTEGER) ||
            (type == Types.NUMERIC) ||
            (type == Types.REAL) ||
            (type == Types.SMALLINT) || 
            (type == Types.TINYINT))
            return false;
        else
/*          CHAR 
            DATE 
            LONGVARCHAR 
            NULL 
            TIME 
            TIMESTAMP 
            VARCHAR 
*/
            return true;
    }
}
